package simpleex.ui;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpRequest.BodyPublishers;
import java.net.http.HttpResponse;
import java.util.Collection;
import java.util.Collections;
import simpleex.core.LatLong;
import simpleex.core.LatLongs;

/*
@startuml
class FxAppController
class LatLongs
class BorderPane
class "ListView<LatLong>" as ListView
class "fxmapcontrol.MapBase" as MapBase

BorderPane *--> ListView: "left"
BorderPane *--> MapBase: "center"

FxAppController --> LatLongs: "latLongs"
FxAppController --> MapBase: "mapView"
FxAppController --> ListView: "locationListView"
@enduml
 */

/**
 * Data access object using rest.
 * @author hal
 *
 */
public class RestLatLongsDataAccess implements LatLongsDataAccess {

  private String baseUrlString;

  private final ObjectMapper objectMapper;

  public RestLatLongsDataAccess(final ObjectMapper objectMapper) {
    this.objectMapper = objectMapper;
  }

  protected ObjectMapper getObjectMapper() {
    return objectMapper;
  }

  private URI getRequestUri(final String path) {
    try {
      return new URI(baseUrlString + path);
    } catch (final URISyntaxException e) {
      throw new IllegalArgumentException(e);
    }
  }

  @Override
  public Collection<LatLong> getAllLatLongs() {
    final HttpRequest request = HttpRequest.newBuilder(getRequestUri(""))
      .GET()
      .build();
      try {
        final HttpResponse<InputStream> response = HttpClient.newBuilder()
        .build()
        .send(request, HttpResponse.BodyHandlers.ofInputStream());
        return getObjectMapper().readValue(response.body(), LatLongs.class).toList();
      } catch (JsonParseException | JsonMappingException e) {
      } catch (IOException | InterruptedException e) {
      }
    return Collections.emptyList();
  }

  @Override
  public LatLong getLatLong(final int num) {
    final HttpRequest request = HttpRequest.newBuilder(getRequestUri("/" + num))
        .GET()
        .build();
      try {
        final HttpResponse<InputStream> response = HttpClient.newBuilder()
        .build()
        .send(request, HttpResponse.BodyHandlers.ofInputStream());
        return getObjectMapper().readValue(response.body(), LatLong.class);
      } catch (JsonParseException | JsonMappingException e) {
      } catch (IOException | InterruptedException e) {
      }
    return null;
  }

  @Override
  public void setLatLong(final int index, final LatLong latLong) {
    try {
      final HttpRequest request = HttpRequest.newBuilder(getRequestUri("/" + index))
          .PUT(BodyPublishers.ofString(getObjectMapper().writeValueAsString(latLong)))
          .build();
        final HttpResponse<InputStream> response = HttpClient.newBuilder()
            .build()
            .send(request, HttpResponse.BodyHandlers.ofInputStream());
        final int realIndex = getObjectMapper().readValue(response.body(), Integer.class);
        if (realIndex < 0) {
          throw new IndexOutOfBoundsException(realIndex);
        }
    } catch (final JsonProcessingException e) {
      throw new RuntimeException(e);
    } catch (IOException | InterruptedException e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public int addLatLong(final LatLong latLong) {
    try {
      final HttpRequest request = HttpRequest.newBuilder(getRequestUri(""))
          .POST(BodyPublishers.ofString(getObjectMapper().writeValueAsString(latLong)))
          .build();
        final HttpResponse<InputStream> response = HttpClient.newBuilder()
            .build()
            .send(request, HttpResponse.BodyHandlers.ofInputStream());
        final int realIndex = getObjectMapper().readValue(response.body(), Integer.class);
        if (realIndex < 0) {
          throw new IndexOutOfBoundsException(realIndex);
        }
    } catch (final JsonProcessingException e) {
      throw new RuntimeException(e);
    } catch (IOException | InterruptedException e) {
      throw new RuntimeException(e);
    }
    return 0;
  }
}
